/*
calculate arctangent y/x in the range of -Pi,+Pi
any arguments (-32767...32767) are permited.
if the arguments are 0,0 -> 0 returned
-Pi corresponds to -32768, Pi corresponds to 32768
Max. error is 0.012 degrees, avg. error is 0.006 degrees
Using mul and div operations
*/

#include "mul.h"
#include "div.h"


int i_atan2(int y,int x)
{

const static int atan_table[33] =
 {
 0,326,651,975,1297,1617,1933,2246,
 2555,2860,3159,3453,3742,4025,4302,4572,
 4836,5094,5344,5589,5826,6058,6282,6500,
 6712,6917,7117,7310,7498,7679,7856,8026,
 8192
 };

unsigned char j;
int ax,ay,tmp,result;

if(y==0)
 {
 if(x<0) return -32768;
 return 0;
 }

if(x==0)
 {
 if(y>0) return 16384;
 return -16384;
 }

ax = x;
if(ax<0) ax = -ax;

ay = y;
if(ay<0) ay = -ay;

if(ax==ay) result = 8192;
else
 {
 tmp = 0;
 if(ax<ay)
  {
  tmp = ax;
  ax = ay;
  ay = tmp;
  }

 ay=div((((long int)ay)<<16)>>1,ax); // ay = 0...32767

 j=(unsigned char)(ay>>10);  // integer  0...31
 ax=(ay&0x3ff)<<5; // fraction (0...1023)<<5
 result=atan_table[j]+(int)(mul((atan_table[j+1]-atan_table[j])<<1,ax)>>16);

 if(tmp!=0) result = 16384 - result;

 }

if(x<0) result = 32768 - result;
if(y<0) result = - result;

return result;
}
